@@ -31,14 +31,11 @@
 #include "siproxd.h"
 #include "log.h"
 
-static char const ident[]="$Id: accessctl.c,v 1.18 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: accessctl.c,v 1.19 2008/05/17 15:55:03 hb9xar Exp $";
 
 /* configuration storage */
 struct siproxd_config configuration;
 
-/* prototypes used locally only */
-int process_aclist (char *aclist, struct sockaddr_in from);
-
 
 /*
  * verifies the from address agains the access lists
