@@ -24,30 +24,91 @@
 #include <stdlib.h>
 #include <string.h>
 #include <netinet/in.h>
+#include <arpa/inet.h>
 
 #include <osipparser2/osip_parser.h>
 
 #include "siproxd.h"
+#include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: plugin_shortdial.c,v 1.7 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: plugin_shortdial.c,v 1.10 2008/02/02 17:10:58 hb9xar Exp $";
 
-/* configuration storage */
+/* Plug-in identification */
+static char name[]="plugin_shortdial";
+static char desc[]="Handles Dial shortcuts as defined in config file";
+
+/* global configuration storage - required for config file location */
 extern struct siproxd_config configuration;
 
+
+/* plugin configuration storage */
+static struct plugin_config {
+   char *shortdial_akey;
+   stringa_t shortdial_entry;
+} plugin_cfg;
+/* Instructions for config parser */
+static cfgopts_t plugin_cfg_opts[] = {
+   { "plugin_shortdial_akey",   TYP_STRING, &plugin_cfg.shortdial_akey },
+   { "plugin_shortdial_entry",  TYP_STRINGA,&plugin_cfg.shortdial_entry },
+   {0, 0, 0}
+};
+
+
 /* local prototypes */
 static int plugin_shortdial_redirect(sip_ticket_t *ticket, int shortcut_no);
+static int plugin_shortdial(sip_ticket_t *ticket);
 
 
+/* 
+ * Plugin API functions code
+ */
+/* Initialization */
+int  plugin_init(plugin_def_t *plugin_def) {
+   plugin_def->api_version=SIPROXD_API_VERSION;
+   plugin_def->name=name;
+   plugin_def->desc=desc;
+   plugin_def->exe_mask=PLUGIN_DETERMINE_TARGET;
+
+   /* read the config file */
+   if (read_config(configuration.configfile,
+                   configuration.config_search,
+                   plugin_cfg_opts, name) == STS_FAILURE) {
+      ERROR("Plugin '%s': could not load config file", name);
+      return STS_FAILURE;
+   }
+
+   return STS_SUCCESS;
+}
+
+/* Processing */
+int  plugin_process(int stage, sip_ticket_t *ticket){
+   int sts;
+   sts=plugin_shortdial(ticket);
+   return sts;
+}
+
+/* De-Initialization */
+int  plugin_end(plugin_def_t *plugin_def){
+   return STS_SUCCESS;
+}
+
+
+/*
+ * Workload code
+ */
+
 /* returns STS_SIP_SENT if processing is to be terminated,
  * otherwise STS_SUCCESS (go on with processing) */
 /* code (entry point) */
-int plugin_shortdial(sip_ticket_t *ticket) {
+static int plugin_shortdial(sip_ticket_t *ticket) {
    int sts=STS_SUCCESS;
    osip_uri_t *req_url;
    int  shortcut_no=0;
 
-   if (!ticket || !ticket->sipmsg) return STS_FAILURE;
+   /* plugin loaded and not configured, return with success */
+   if (plugin_cfg.shortdial_akey==NULL) return STS_SUCCESS;
+   if (plugin_cfg.shortdial_entry.used==0) return STS_SUCCESS;
 
    DEBUGC(DBCLASS_PLUGIN,"plugin entered");
    req_url=osip_message_get_uri(ticket->sipmsg);
@@ -64,23 +125,23 @@ int plugin_shortdial(sip_ticket_t *ticket) {
    /* REQ URI with username must exist, length as defined in config,
     * shortdial must be enabled and short dial key must match */
    if (!req_url || !req_url->username ||
-       !configuration.pi_shortdial_akey ||
-       (strlen(req_url->username) != strlen(configuration.pi_shortdial_akey)) ||
-       (req_url->username[0] != configuration.pi_shortdial_akey[0]))
+       !plugin_cfg.shortdial_akey ||
+       (strlen(req_url->username) != strlen(plugin_cfg.shortdial_akey)) ||
+       (req_url->username[0] != plugin_cfg.shortdial_akey[0]))
       return STS_SUCCESS; /* ignore */
 
    shortcut_no = atoi(&(req_url->username[1]));
    if ((shortcut_no <= 0) || (shortcut_no >= LONG_MAX)) return STS_SUCCESS; /* not a number */
 
    /* requested number is not defined (out of range) */
-   if (shortcut_no > configuration.pi_shortdial_entry.used) {
+   if (shortcut_no > plugin_cfg.shortdial_entry.used) {
       DEBUGC(DBCLASS_PLUGIN, "shortdial: shortcut %i > available shortcuts (%i)",
-            shortcut_no, configuration.pi_shortdial_entry.used);
+            shortcut_no, plugin_cfg.shortdial_entry.used);
       return STS_SUCCESS;
    }
 
    /* requested number is not defined (empty) */
-   if (!configuration.pi_shortdial_entry.string[shortcut_no-1]) {
+   if (!plugin_cfg.shortdial_entry.string[shortcut_no-1]) {
       DEBUGC(DBCLASS_PLUGIN, "shortdial: shortcut %i empty", shortcut_no);
       return STS_SUCCESS;
    }
@@ -105,7 +166,7 @@ int plugin_shortdial(sip_ticket_t *ticket) {
 }
 
 
-/* private code */
+/* private plugin code */
 static int plugin_shortdial_redirect(sip_ticket_t *ticket, int shortcut_no) {
    osip_uri_t *to_url=ticket->sipmsg->to->url;
    char *to_user=to_url->username;
@@ -114,7 +175,7 @@ static int plugin_shortdial_redirect(sip_ticket_t *ticket, int shortcut_no) {
    size_t len;
    osip_contact_t *contact = NULL;
 
-   new_to_user=configuration.pi_shortdial_entry.string[shortcut_no-1];
+   new_to_user=plugin_cfg.shortdial_entry.string[shortcut_no-1];
    if (!new_to_user) return STS_SUCCESS;
 
    DEBUGC(DBCLASS_PLUGIN,"redirect: redirecting [%s]->[%s]",
