@@ -34,9 +34,10 @@
 #include <osipparser2/sdp_message.h>
 
 #include "siproxd.h"
+#include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: proxy.c,v 1.109 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: proxy.c,v 1.113 2008/02/24 18:29:31 hb9xar Exp $";
 
 /* configuration storage */
 extern struct siproxd_config configuration;	/* defined in siproxd.c */
@@ -105,63 +106,11 @@ int proxy_request (sip_ticket_t *ticket) {
     * figure out whether this is an incoming or outgoing request
     * by doing a lookup in the registration table.
     */
-    sip_find_direction(ticket, &i);
-    type = ticket->direction;
-
-   /*
-    * logging of passing calls
-    */
-/*&&&& this should be moved to its own logging plugin */
-   if (configuration.log_calls) {
-      osip_uri_t *from_url = NULL;
-      osip_uri_t *to_url   = NULL;
-      char *to_username =NULL;
-      char *to_host = NULL;
-      char *from_username =NULL;
-      char *from_host = NULL;
-      char *call_type = NULL;
-
-      /* From: 1st preference is From header, then try contact header */
-      if (request->from->url) {
-         from_url = request->from->url;
-      } else {
-         from_url = (osip_uri_t *)osip_list_get(&(request->contacts), 0);
-      }
-
-      to_url = request->to->url;
-
-      if (to_url) {
-         to_username = to_url->username;
-         to_host = to_url->host;
-      }
-
-      if (from_url) {
-         from_username = from_url->username;
-         from_host = from_url->host;
-      }
-
-      /* INVITE */
-      if (MSG_IS_INVITE(request)) {
-         if (type==REQTYP_INCOMING) call_type="Incoming";
-         else call_type="Outgoing";
-      /* BYE / CANCEL */
-      } else if (MSG_IS_ACK(request)) {
-         call_type="ACK";
-      } else if (MSG_IS_BYE(request) || MSG_IS_CANCEL(request)) {
-         call_type="Ending";
-      }
-
-      if (call_type) {
-         INFO("%s Call: %s@%s -> %s@%s",
-              call_type,
-              from_username ? from_username: "*NULL*",
-              from_host     ? from_host    : "*NULL*",
-              to_username   ? to_username  : "*NULL*",
-              to_host       ? to_host      : "*NULL*");
-      }
-
-   } /* log_calls */
+   sip_find_direction(ticket, &i);
+   type = ticket->direction;
 
+   /* Call Plugins for stage: PLUGIN_PRE_PROXY */
+   sts = call_plugins(PLUGIN_PRE_PROXY, ticket);
 
    /*
     * RFC 3261, Section 16.6 step 1
@@ -288,6 +237,11 @@ sts=sip_obscure_callid(ticket);
  * How about "408 Request Timeout" ?
  *
  */
+      /* Call Plugins for stage: PLUGIN_PROXY_UNK */
+      sts = call_plugins(PLUGIN_PROXY_UNK, ticket);
+      /*&&& we might want have the possibility for a plugin to generate
+            and send a response, similar an in stage PLUGIN_DETERMINE_TARGET */
+
       sip_gen_response(ticket, 408 /* Request Timeout */);
 
       return STS_FAILURE;
@@ -406,6 +360,10 @@ sts=sip_obscure_callid(ticket);
       DEBUGC(DBCLASS_PROXY, "proxy_request: have Route header to %s:%i",
              utils_inet_ntoa(sendto_addr), port);
 #else
+   /*
+    * Route present?
+    * If so, fetch address from topmost Route: header and remove it.
+    */
    if ((type == REQTYP_OUTGOING) && 
               (!osip_list_eol(&(request->routes), 0))) {
       sts=route_determine_nexthop(ticket, &sendto_addr, &port);
@@ -419,16 +377,17 @@ sts=sip_obscure_callid(ticket);
        (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
       DEBUGC(DBCLASS_PROXY, "proxy_request: have outbound proxy %s:%i",
              utils_inet_ntoa(sendto_addr), port);
-   /*
-    * Route present?
-    * If so, fetch address from topmost Route: header and remove it.
-    */
 #endif
    /*
     * destination from SIP URI
     */
    } else {
       /* get the destination from the SIP URI */
+/*&&&& Here, the SRV record lookup magic must go.
+In a first implementation we may just try to get the lowest priority,
+max weighted '_sip._udp.domain' entry and port number.
+No load balancing and no failover are supported with this.
+&&&*/
       sts = get_ip_by_host(url->host, &sendto_addr);
       if (sts == STS_FAILURE) {
          DEBUGC(DBCLASS_PROXY, "proxy_request: cannot resolve URI [%s]",
@@ -469,6 +428,9 @@ sts=sip_obscure_callid(ticket);
    */
   /* not necessary, already in message and we do not support TCP */
 
+   /* Call Plugins for stage: PLUGIN_POST_PROXY */
+   sts = call_plugins(PLUGIN_POST_PROXY, ticket);
+
   /*
    * RFC 3261, Section 16.6 step 10
    * Proxy Behavior - Forward the new request
@@ -479,8 +441,7 @@ sts=sip_obscure_callid(ticket);
       return STS_FAILURE;
    }
 
-   sipsock_send(sendto_addr, port, ticket->protocol,
-                buffer, buflen); 
+   sipsock_send(sendto_addr, port, ticket->protocol, buffer, buflen);
    osip_free (buffer);
 
   /*
@@ -548,8 +509,11 @@ int proxy_response (sip_ticket_t *ticket) {
     * figure out if this is an request coming from the outside
     * world to one of our registered clients
     */
-    sip_find_direction(ticket, NULL);
-    type = ticket->direction;
+   sip_find_direction(ticket, NULL);
+   type = ticket->direction;
+
+   /* Call Plugins for stage: PLUGIN_PRE_PROXY */
+   sts = call_plugins(PLUGIN_PRE_PROXY, ticket);
 
 /*
  * ok, we got a response that we are allowed to process.
@@ -679,6 +643,9 @@ sts=sip_obscure_callid(ticket);
       DEBUGC(DBCLASS_PROXY, "response from/to unregistered UA (%s@%s)",
 	   response->from->url->username? response->from->url->username:"*NULL*",
 	   response->from->url->host? response->from->url->host : "*NULL*");
+
+      /* Call Plugins for stage: PLUGIN_PROXY_UNK */
+      sts = call_plugins(PLUGIN_PROXY_UNK, ticket);
       return STS_FAILURE;
    }
 
@@ -705,7 +672,7 @@ sts=sip_obscure_callid(ticket);
 /*&&&& priority probably should be:
  * 1) Route header
  * 2) fixed outbound proxy
- * 3) SIP URI
+ * 3) Via header
  */
    /*
     * check if we need to send to an outbound proxy
@@ -730,6 +697,10 @@ sts=sip_obscure_callid(ticket);
       DEBUGC(DBCLASS_PROXY, "proxy_response: have Route header to %s:%i",
              utils_inet_ntoa(sendto_addr), port);
 #else
+   /*
+    * Route present?
+    * If so, fetch address from topmost Route: header and remove it.
+    */
    if ((type == RESTYP_OUTGOING) && 
               (!osip_list_eol(&(response->routes), 0))) {
       sts=route_determine_nexthop(ticket, &sendto_addr, &port);
@@ -743,10 +714,6 @@ sts=sip_obscure_callid(ticket);
        (sip_find_outbound_proxy(ticket, &sendto_addr, &port) == STS_SUCCESS)) {
       DEBUGC(DBCLASS_PROXY, "proxy_response: have outbound proxy %s:%i",
              utils_inet_ntoa(sendto_addr), port);
-   /*
-    * Route present?
-    * If so, fetch address from topmost Route: header and remove it.
-    */
 #endif
    } else {
       /* get target address and port from VIA header */
@@ -771,14 +738,19 @@ sts=sip_obscure_callid(ticket);
       }
    }
 
+   /* Call Plugins for stage: PLUGIN_POST_PROXY */
+   sts = call_plugins(PLUGIN_POST_PROXY, ticket);
+
+  /*
+   * Proxy Behavior - Forward the response
+   */
    sts = sip_message_to_str(response, &buffer, &buflen);
    if (sts != 0) {
       ERROR("proxy_response: sip_message_to_str failed");
       return STS_FAILURE;
    }
 
-   sipsock_send(sendto_addr, port, ticket->protocol,
-                buffer, buflen); 
+   sipsock_send(sendto_addr, port, ticket->protocol, buffer, buflen);
    osip_free (buffer);
    return STS_SUCCESS;
 }
