@@ -42,7 +42,7 @@
 #include "digcalc.h"
 #include "log.h"
 
-static char const ident[]="$Id: sip_utils.c,v 1.56 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: sip_utils.c,v 1.59 2008/07/22 11:17:49 hb9xar Exp $";
 
 
 /* configuration storage */
@@ -773,13 +773,13 @@ int  sip_calculate_branch_id (sip_ticket_t *ticket, char *id) {
       if (strncmp(param->gvalue, magic_cookie,
                   strlen(magic_cookie))==0) {
          /* calculate MD5 hash */
-         MD5_CTX Md5Ctx;
+         osip_MD5_CTX Md5Ctx;
          HASH HA1;
 
-         MD5Init(&Md5Ctx);
-         MD5Update(&Md5Ctx, param->gvalue,
-                   strlen(param->gvalue));
-         MD5Final(HA1, &Md5Ctx);
+         osip_MD5Init(&Md5Ctx);
+         osip_MD5Update(&Md5Ctx, (unsigned char*)param->gvalue,
+                        strlen(param->gvalue)); 
+         osip_MD5Final(HA1, &Md5Ctx);
          CvtHex(HA1, hashstring);
 
          DEBUGC(DBCLASS_BABBLE, "existing branch -> branch hash [%s]",
@@ -799,53 +799,55 @@ int  sip_calculate_branch_id (sip_ticket_t *ticket, char *id) {
     */
    if (hashstring[0] == '\0') {
       /* calculate MD5 hash */
-      MD5_CTX Md5Ctx;
+      osip_MD5_CTX Md5Ctx;
       HASH HA1;
       char *tmp;
 
-      MD5Init(&Md5Ctx);
+      osip_MD5Init(&Md5Ctx);
 
       /* topmost via */
       osip_via_to_str(via, &tmp);
       if (tmp) {
-         MD5Update(&Md5Ctx, tmp, strlen(tmp));
+         osip_MD5Update(&Md5Ctx, (unsigned char*)tmp, strlen(tmp));
          osip_free(tmp);
       }
      
       /* Tag in To header */
       osip_to_get_tag(sip_msg->to, &param);
       if (param && param->gvalue) {
-         MD5Update(&Md5Ctx, param->gvalue, strlen(param->gvalue));
+         osip_MD5Update(&Md5Ctx, (unsigned char*)param->gvalue,
+	                strlen(param->gvalue));
       }
 
       /* Tag in From header */
       osip_from_get_tag(sip_msg->from, &param);
       if (param && param->gvalue) {
-         MD5Update(&Md5Ctx, param->gvalue, strlen(param->gvalue));
+         osip_MD5Update(&Md5Ctx, (unsigned char*)param->gvalue,
+	                strlen(param->gvalue));
       }
 
       /* Call-ID */
       call_id = osip_message_get_call_id(sip_msg);
       osip_call_id_to_str(call_id, &tmp);
       if (tmp) {
-         MD5Update(&Md5Ctx, tmp, strlen(tmp));
+         osip_MD5Update(&Md5Ctx, (unsigned char*)tmp, strlen(tmp));
          osip_free(tmp);
       }
 
       /* CSeq number (but not method) */
       tmp = osip_cseq_get_number(sip_msg->cseq);
       if (tmp) {
-         MD5Update(&Md5Ctx, tmp, strlen(tmp));
+         osip_MD5Update(&Md5Ctx, (unsigned char*)tmp, strlen(tmp));
       }
  
       /* Request URI */
       osip_uri_to_str(sip_msg->req_uri, &tmp);
       if (tmp) {
-         MD5Update(&Md5Ctx, tmp, strlen(tmp));
+         osip_MD5Update(&Md5Ctx, (unsigned char*)tmp, strlen(tmp));
          osip_free(tmp);
       }
 
-      MD5Final(HA1, &Md5Ctx);
+      osip_MD5Final(HA1, &Md5Ctx);
       CvtHex(HA1, hashstring);
 
       DEBUGC(DBCLASS_BABBLE, "non-existing branch -> branch hash [%s]",
@@ -973,9 +975,9 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
    from=&ticket->from;
    request=ticket->sipmsg;
    response=ticket->sipmsg;
-   type = 0;
+   type = DIRTYP_UNKNOWN;
 
-   ticket->direction = 0;
+   ticket->direction = DIRTYP_UNKNOWN;
 
    /*
     * did I receive the telegram from a REGISTERED host?
@@ -1004,7 +1006,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
     * is the telegram directed to an internally registered host?
     * -> it must be an INCOMING request
     */
-   if (type == 0) {
+   if (type == DIRTYP_UNKNOWN) {
       for (i=0; i<URLMAP_SIZE; i++) {
          if (urlmap[i].active == 0) continue;
          /* RFC3261:
@@ -1046,7 +1048,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
             }
          } /* is request */
       } /* for i */
-   } /* if type == 0 */
+   } /* if type == DIRTYP_UNKNOWN */
 
    if (MSG_IS_RESPONSE(ticket->sipmsg)) {
       /* &&&& Open Issue &&&&
@@ -1057,7 +1059,8 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
          Maybe I should put in a 'siproxd' ftag value to recognize it as a header
          inserted by myself
       */
-      if ((type == 0) && (!osip_list_eol(&(response->vias), 0))) {
+      if ((type == DIRTYP_UNKNOWN) && 
+          (!osip_list_eol(&(response->vias), 0))) {
          osip_via_t *via;
          struct in_addr addr_via, addr_myself;
          int port_via, port_ua;
@@ -1103,7 +1106,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
                }
             } /* for i */
          }
-      } /* if type == 0 */
+      } /* if type == DIRTYP_UNKNOWN */
    } /* is response */
 
    /*
@@ -1126,7 +1129,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
    }
 
 
-   if (type == 0) {
+   if (type == DIRTYP_UNKNOWN) {
       DEBUGC(DBCLASS_SIP, "sip_find_direction: unable to determine "
                           "direction of SIP packet");
       return STS_FAILURE;
@@ -1150,7 +1153,7 @@ int  sip_find_direction(sip_ticket_t *ticket, int *urlidx) {
  * RETURNS
  *	STS_SUCCESS on success
  */
-int  sip_fixup_asterisk(char *buff, int *buflen) {
+int  sip_fixup_asterisk(char *buff, size_t *buflen) {
    char *alert_info_ptr=NULL;
    /*
     * Check for Asterisk UA string
