@@ -40,10 +40,54 @@
 #include "plugins.h"
 #include "log.h"
 
-static char const ident[]="$Id: siproxd.c,v 1.73 2008/01/19 16:07:14 hb9xar Exp $";
+static char const ident[]="$Id: siproxd.c,v 1.78 2008/02/04 20:36:48 hb9xar Exp $";
 
 /* configuration storage */
 struct siproxd_config configuration;
+/* instructions for config parser */
+static cfgopts_t main_cfg_opts[] = {
+   { "debug_level",         TYP_INT4,   &configuration.debuglevel },
+   { "debug_port",          TYP_INT4,   &configuration.debugport },
+   { "sip_listen_port",     TYP_INT4,   &configuration.sip_listen_port },
+   { "daemonize",           TYP_INT4,   &configuration.daemonize },
+   { "silence_log",         TYP_INT4,   &configuration.silence_log },
+   { "if_inbound",          TYP_STRING, &configuration.inbound_if },
+   { "if_outbound",         TYP_STRING, &configuration.outbound_if },
+   { "host_outbound",       TYP_STRING, &configuration.outbound_host },
+   { "rtp_port_low",        TYP_INT4,   &configuration.rtp_port_low },
+   { "rtp_port_high",       TYP_INT4,   &configuration.rtp_port_high },
+   { "rtp_timeout",         TYP_INT4,   &configuration.rtp_timeout },
+   { "rtp_proxy_enable",    TYP_INT4,   &configuration.rtp_proxy_enable },
+   { "rtp_dscp",            TYP_INT4,   &configuration.rtp_dscp },
+   { "rtp_input_dejitter",  TYP_INT4,   &configuration.rtp_input_dejitter },
+   { "rtp_output_dejitter", TYP_INT4,   &configuration.rtp_output_dejitter },
+   { "user",                TYP_STRING, &configuration.user },
+   { "chrootjail",          TYP_STRING, &configuration.chrootjail },
+   { "hosts_allow_reg",     TYP_STRING, &configuration.hosts_allow_reg },
+   { "hosts_allow_sip",     TYP_STRING, &configuration.hosts_allow_sip },
+   { "hosts_deny_sip",      TYP_STRING, &configuration.hosts_deny_sip },
+   { "hosts_deny_sip",      TYP_STRING, &configuration.hosts_deny_sip },
+   { "proxy_auth_realm",    TYP_STRING, &configuration.proxy_auth_realm },
+   { "proxy_auth_passwd",   TYP_STRING, &configuration.proxy_auth_passwd },
+   { "proxy_auth_pwfile",   TYP_STRING, &configuration.proxy_auth_pwfile },
+   { "mask_host",           TYP_STRINGA,&configuration.mask_host },
+   { "masked_host",         TYP_STRINGA,&configuration.masked_host },
+   { "outbound_proxy_host", TYP_STRING, &configuration.outbound_proxy_host },
+   { "outbound_proxy_port", TYP_INT4,   &configuration.outbound_proxy_port },
+   { "outbound_domain_name",TYP_STRINGA,&configuration.outbound_proxy_domain_name },
+   { "outbound_domain_host",TYP_STRINGA,&configuration.outbound_proxy_domain_host },
+   { "outbound_domain_port",TYP_STRINGA,&configuration.outbound_proxy_domain_port },
+   { "registration_file",   TYP_STRING, &configuration.registrationfile },
+   { "pid_file",            TYP_STRING, &configuration.pid_file },
+   { "default_expires",     TYP_INT4,   &configuration.default_expires },
+   { "autosave_registrations",TYP_INT4, &configuration.autosave_registrations },
+   { "ua_string",           TYP_STRING, &configuration.ua_string },
+   { "use_rport",           TYP_INT4,   &configuration.use_rport },
+   { "obscure_loops",       TYP_INT4,   &configuration.obscure_loops },
+   { "plugindir",           TYP_STRING, &configuration.plugin_dir },
+   { "load_plugin",         TYP_STRINGA,&configuration.load_plugin },
+   {0, 0, 0}
+};
 
 /* Global File instance on pw file */
 FILE *siproxd_passwordfile;
@@ -185,7 +229,13 @@ int main (int argc, char *argv[])
    INFO(PACKAGE"-"VERSION"-"BUILDSTR" "UNAME" starting up");
 
    /* read the config file */
-   if (read_config(configfile, config_search) == STS_FAILURE) exit(1);
+   if (read_config(configfile, config_search, main_cfg_opts, "") == STS_FAILURE) {
+      exit(1);
+   }
+   /* remember where config file is located, so the plugins
+    * can load it as well - if required */
+   configuration.configfile = strdup(configfile);
+   configuration.config_search = config_search;
 
    /* if a debug level > 0 has been given on the commandline use its
       value and not what is in the config file */
@@ -236,6 +286,14 @@ int main (int argc, char *argv[])
       INFO("daemonized, pid=%i", getpid());
    }
 
+   /* load and initialize the plugins */
+   sts=load_plugins();
+   /* if error, abort siproxd */
+   if (sts != STS_SUCCESS) {
+      ERROR("Error while loading and initializing plug-ins - aborting");
+      exit(1);
+   }
+
    /* prepare for creating PID file */
    if (pidfilename == NULL) pidfilename = configuration.pid_file;
 
@@ -381,6 +439,9 @@ int main (int argc, char *argv[])
        */
       /* NOT IMPLEMENTED */
 
+      /* Call Plugins for stage: PLUGIN_VALIDATE */
+      sts = call_plugins(PLUGIN_VALIDATE, &ticket);
+
       /*
        * RFC 3261, Section 16.3 step 3
        * Proxy Behavior - Request Validation - Max-Forwards check
@@ -445,17 +506,17 @@ int main (int argc, char *argv[])
                    ticket.sipmsg->reason_phrase : "NULL")));
 
       /*********************************
+       * Call Plugins for stage: PLUGIN_DETERMINE_TARGET
        * The message did pass all the
        * tests above and is now ready
        * to be proxied.
-       * Before we do so, we apply some
-       * additional preprocessing
+       * Feed to the plugins. If a plugin decides
+       * to end processing and terminate the ongoing
+       * dialog (STS_SIP_SENT), then just free
+       * the allocated resources.
        *********************************/
-      /* Dial shortcuts */
-      if (configuration.pi_shortdial) {
-         sts = plugin_shortdial(&ticket);
-         if (sts == STS_SIP_SENT) goto end_loop;
-      }
+      sts = call_plugins(PLUGIN_DETERMINE_TARGET, &ticket);
+      if (sts == STS_SIP_SENT) goto end_loop;
 
 
       /*********************************
@@ -583,6 +644,9 @@ int main (int argc, char *argv[])
       }
    }
 
+   /* unload the plugins */
+   unload_plugins();
+
    /* END */
    log_end();
    return 0;
