@@ -18,7 +18,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 */
 
-/* $Id: siproxd.h,v 1.79 2008/01/19 16:07:14 hb9xar Exp $ */
+/* $Id: siproxd.h,v 1.86 2008/07/22 11:17:49 hb9xar Exp $ */
 
 #ifdef DMALLOC
  #include <dmalloc.h>
@@ -43,6 +43,7 @@ struct urlmap_s {
  */
 
 
+
 /*
  * Array of strings - used within configuration store
  */
@@ -53,9 +54,12 @@ typedef struct {
 } stringa_t;
 
 /*
- * configuration option table
+ * global configuration option table
  */
 struct siproxd_config {
+   char *configfile;
+   int   config_search;
+   /* everything below here will be filled according to the config file */
    unsigned int debuglevel;
    int debugport;
    char *inbound_if;
@@ -87,18 +91,25 @@ struct siproxd_config {
    stringa_t outbound_proxy_domain_host;
    stringa_t outbound_proxy_domain_port;
    char *registrationfile;
-   int  log_calls;
    char *pid_file;
    int  default_expires;
    int  autosave_registrations;
-   int  pi_shortdial;
-   char *pi_shortdial_akey;
-   stringa_t pi_shortdial_entry;
    char *ua_string;
    int   use_rport;
    int   obscure_loops;
+   char *plugin_dir;
+   stringa_t load_plugin;
 };
 
+/*
+ * control structure for config file parser
+ */
+typedef struct {
+   char *keyword;
+   enum type {TYP_INT4, TYP_STRING, TYP_FLOAT, TYP_STRINGA} type;
+   void *dest;
+} cfgopts_t;
+
 /*
  * SIP ticket
  */
@@ -109,6 +120,7 @@ typedef struct {
 #define PROTO_UDP  1
 #define PROTO_TCP  2
    int protocol;		/* received by protocol */
+#define DIRTYP_UNKNOWN		0
 #define REQTYP_INCOMING		1
 #define REQTYP_OUTGOING		2
 #define RESTYP_INCOMING		3
@@ -141,7 +153,7 @@ int sipsock_listen(void);						/*X*/
 int sipsock_wait(void);
 int sipsock_read(void *buf, size_t bufsize,
                  struct sockaddr_in *from, int *protocol);
-int sipsock_send(struct in_addr addr, int port,	int protocol,			/*X*/
+int sipsock_send(struct in_addr addr, int port,	int protocol,		/*X*/
                  char *buffer, size_t size);
 int sockbind(struct in_addr ipaddr, int localport, int errflg);
 
@@ -194,12 +206,12 @@ int  sip_calculate_branch_id (sip_ticket_t *ticket, char *id);		/*X*/
 int  sip_find_outbound_proxy(sip_ticket_t *ticket, struct in_addr *addr,
                              int *port);				/*X*/
 int  sip_find_direction(sip_ticket_t *ticket, int *urlidx);		/*X*/
-int  sip_fixup_asterisk(char *buff, int *buflen);			/*X*/
+int  sip_fixup_asterisk(char *buff, size_t *buflen);			/*X*/
 int  sip_obscure_callid(sip_ticket_t *ticket);				/*X*/
 
 /* readconf.c */
-int read_config(char *name, int search);				/*X*/
-int make_default_config(void);						/*X*/
+int  read_config(char *name, int search, cfgopts_t cfgopts[], char *filter); /*X*/
+int  make_default_config(void);						/*X*/
 
 /* rtpproxy.c */
 int  rtpproxy_init( void );						/*X*/
@@ -212,6 +224,7 @@ int  rtp_stop_fwd (osip_call_id_t *callid, int direction);		/*X*/
 
 /* accessctl.c */
 int  accesslist_check(struct sockaddr_in from);
+int  process_aclist (char *aclist, struct sockaddr_in from);
 
 /* security.c */
 int  security_check_raw(char *sip_buffer, size_t size);			/*X*/
@@ -220,7 +233,7 @@ int  security_check_sip(sip_ticket_t *ticket);				/*X*/
 /* auth.c */
 int  authenticate_proxy(osip_message_t *sipmsg);			/*X*/
 int  auth_include_authrq(osip_message_t *sipmsg);			/*X*/
-void CvtHex(char *hash, char *hashstring);
+void CvtHex(unsigned char *hash, unsigned char *hashstring);
 
 /* fwapi.c */
 int fwapi_start_rtp(int rtp_direction,
@@ -236,6 +249,10 @@ int sip_message_to_str(osip_message_t * sip,   char **dest,     size_t *len);
 int sip_body_to_str(const osip_body_t * body,  char **dest,     size_t *len);
 int sip_message_set_body(osip_message_t * sip, const char *buf, size_t len);
 
+/* plugins.c */
+int load_plugins (void);
+int call_plugins(int stage, sip_ticket_t *ticket);
+int unload_plugins(void);
 
 /*
  * some constant definitions
@@ -258,6 +275,7 @@ int sip_message_set_body(osip_message_t * sip, const char *buf, size_t len);
 #define RTP_BUFFER_SIZE	1520	/* max size of an RTP frame		*/
 				/* (assume approx one Ethernet MTU)	*/
 
+#define PATH_STRING_SIZE 256	/* max size of an file path		*/
 #define URL_STRING_SIZE	128	/* max size of an URL/URI string	*/
 #define STATUSCODE_SIZE	5	/* size of string representation of status */
 #define DNS_CACHE_SIZE	256	/* number of entries in internal DNS cache */
@@ -271,6 +289,7 @@ int sip_message_set_body(osip_message_t * sip, const char *buf, size_t len);
 #define HOSTNAME_SIZE	128	/* max string length of a hostname	*/
 #define USERNAME_SIZE	128	/* max string length of a username (auth) */
 #define PASSWORD_SIZE	128	/* max string length of a password (auth) */
+#define IPSTRING_SIZE	16	/* stringsize of IP address xxx.xxx.xxx.xxx */
 #define VIA_BRANCH_SIZE	128	/* max string length for via branch param */
 				/* scratch buffer for gethostbyname_r() */
 
@@ -297,7 +316,7 @@ int sip_message_set_body(osip_message_t * sip, const char *buf, size_t len);
 #define STS_FAILURE	1	/* FAILURE				*/
 #define STS_FALSE	1	/* FALSE				*/
 #define STS_NEED_AUTH	1001	/* need authentication			*/
-#define STS_SIP_SENT	2001	/* SIP packet is already sent		*/
+#define STS_SIP_SENT	2001	/* SIP packet is already sent, end of dialog */
 
 /* symbolic direction of data */
 #define DIR_INCOMING	1
